(*  This file contains functions for arithmetic tests on dart boards.  These are
    often used as whole class mental arithmetic exercises.
    version 1  18-09-99
    version 2  10-12-99  RingWidths added.  SectorOuterLabels added.
                         Additional usage messages added.
    version 3  27-02-00  SectorOffset added.
*)

BeginPackage["PrimaryMath`ArithmeticExercises`DartBoards`"
, "PrimaryMath`Utilities`ValueTables`"
, "Utilities`FilterOptions`"] ;

DartBoard::usage =
"DartBoard[opts___Rule] creates a drawing of a dart board with numbers in each segment.
The appearence and content of the dart board is controlled by the options."

Options[DartBoard] =
  { DartBoardEntriesVisible -> True
  , NumberOfRings -> 3
  , NumberOfSectors -> 12
  , RingWidths -> 1
  , SectorFunction -> ((#*2)&)
  , SectorOffset -> 0
  , SectorOuterLabels -> None
  , SectorOuterLabelsMargin -> 0.5
  , SectorRingBackground -> RGBColor[1,1,1]
  , SectorSeeds -> Range[12]
  } ;

DartBoardEntriesVisible::usage =
"DartBoardEntriesVisible is an option of DartBoard which specifies which of the 
numbers on the dart board are to be visible."

NumberOfRings::usage =
"NumberOfRings is an option of DartBoard which specifies the number of rings 
on the DartBoard."

NumberOfSectors::usage =
"NumberOfSectors is an option of DartBoard that specifies the number of sectors
on the DartBoard."

RingWidths::usage =
"RingWidths is an option of DartBoard that specifies the widths of the rings.\n 
e.g \n
RingWidths -> 1 specifies that all rings should have width 1.\n
RingWidths -> { 1 , 2 , 1} specifies that the first inner ring should have width
1, the second ring should have width 2 and the third ring should have width 3."

SectorFunction::usage =
"SectorFunction is an option of DartBoard which specifies the function to be applied 
in computing the numbers in the rings.  The inner ring number in the sector is 
determined by the SectorSeeds option, and the outer ring numbers are determined by
repeatedly applying SectorFunction's value to the inner ring number."

SectorOffset::usage =
"SectorOffset is an option of DartBoard that specifies the offset to be applied
to the position of each sector.  An offset of zero would place the beginning of the
first sector at 12 o'clock.  An offset of 1/2 offsets the sectors by half their 
widths.  Therefore the centre of the last sector would be at 12 o'clock."

SectorOuterLabels::usage =
"SectorOuterLabels is an option of DartBoard which specifies the labels to be
put at the outer edge of the sectors.\n
e.g. SectorOuterLabels->Table[i,{i,10}] specifies that the sectors are to be labelled
1 to 10\n
SectorOuterLabels->None specifies that there are no labels at the outer edge of the
sectors."

SectorOuterLabelsMargin::usage =
"SectorOuterLabelsMargin is an option of DartBoard which specifies the margin between 
the outermost ring and the centre line of the labels defined in SectorOuterLabels."

SectorRingBackground::usage =
"SectorRingBackground is an option of DartBoard which specifies the background 
to be applied to the sectors and rings.  It can take colour and GreyLevel values." 

SectorSeeds::usage =
"SectorSeeds is an option of DartBoard which specifies the numbers in the innermost 
ring.  These numbers are the starting numbers for the computation of the numbers
in the other rings.\n
e.g. SectorSeeds->Range[12] states that the numbers 1 to 12 are used as the seeds.\n
SectorSeeds->None states that there is no seeding and therefore there cannot be
any labelling of the rings in the sectors."

Begin["`Private`"] ;

DartBoard[opts___Rule] :=
  Module[ { entryVisible , i , j , numberList 
      , numberOfRings , numberOfSectors , ringBackground 
      , ringCentre , ringOuterRadius , ringWidths
      , sectorBoundary , sectorOffset 
      , sectorOuterLabels , sectorOuterLabelsMargin 
      , sectorPosition , sectorSeed , sectorSize
      , currentRadius = 0
      }
  , { entryVisible , numberOfRings , numberOfSectors 
    , sectorOffset
    , sectorOuterLabels , sectorOuterLabelsMargin 
    , ringBackground
    , ringWidths 
    , sectorFunction , sectorSeed }
    = { DartBoardEntriesVisible 
      , NumberOfRings , NumberOfSectors , SectorOffset
      , SectorOuterLabels 
      , SectorOuterLabelsMargin , SectorRingBackground 
      , RingWidths , SectorFunction , SectorSeeds } /. 
        { opts } /. 
        Options[DartBoard] ;
    (* If the sectors are seeded, compute the number list *)
    If[ sectorSeed =!= None
    , sectorSeed = ValueTableExpand[sectorSeed,numberOfSectors] ; 
      numberList = 
        Table[ NestList[ sectorFunction , sectorSeed[[i]] , numberOfRings ] 
        , { i , numberOfSectors } ] ;
    ] ;
    
    ringWidths = ValueTableExpand[ringWidths,numberOfRings] ;
    ringOuterRadius = 
      Table[ currentRadius += ringWidths[[j]] , { j , numberOfRings } ] ;
    ringCentre = 
      Join[ { ringOuterRadius[[1]]/2 }
      , Table[ (ringOuterRadius[[j-1]] + ringOuterRadius[[j]])/2
        , { j , 2 , numberOfRings }
        ] 
      ] ;
    ringBackground = ValueTableExpand[ringBackground,numberOfSectors,numberOfRings] ;
    entryVisible = ValueTableExpand[entryVisible,numberOfSectors,numberOfRings] ;
    sectorSize = 2Pi/numberOfSectors ;
    sectorOffset *= sectorSize ;
    sectorBoundary = Table[Pi/2-(i*sectorSize)-sectorOffset,{i,0,numberOfSectors}] ;
    sectorPosition = Drop[sectorBoundary,1]+sectorSize/2 ;
    If[ sectorOuterLabels =!= None
    , sectorOuterLabels = ValueTableExpand[sectorOuterLabels, numberOfSectors ] 
    ] ;
    Show[
      Graphics[{
        Table[ 
          { ringBackground[[i,j]] 
          , Disk[{0,0},ringOuterRadius[[j]],{sectorBoundary[[i+1]],sectorBoundary[[i]]}]
          }
        , { i , numberOfSectors } , { j , numberOfRings , 1 , -1 }
        ]
      , Table[ Circle[{0,0},ringOuterRadius[[j]]] , { j , numberOfRings } ]
      , Table[ 
          Line[
            { {0,0}
            , Last[ringOuterRadius]*{Cos[sectorBoundary[[i]]],Sin[sectorBoundary[[i]]]}}
          ]
        , { i , numberOfSectors }
        ]
      , If[ sectorSeed === None
        , { }
        , Table[ 
            Text[
              VisibleString[ numberList[[i,j]] , entryVisible[[i,j]] ]
            , ringCentre[[j]]* {Cos[sectorPosition[[i]]],Sin[sectorPosition[[i]]]}
            ]
          , { i , numberOfSectors } , { j , numberOfRings }
          ]
        ]
      , If[ sectorOuterLabels === None
        , { }
        , Table[
            Text[ sectorOuterLabels[[i]]  
            ,   (Last[ringOuterRadius]+sectorOuterLabelsMargin) 
              * {Cos[sectorPosition[[i]]],Sin[sectorPosition[[i]]]}
            ]
          , { i , numberOfSectors }
          ]
        ]
      }]
    , AspectRatio->1 , FilterOptions[Graphics,opts]
    ]
  ] ;

End[] ;

EndPackage[] ;