(*  This file contains functions for creating and manipulating number lines. 
    version 1  20-08-00
*)

BeginPackage["PrimaryMath`ArithmeticExercises`NumberLines`"
, "PrimaryMath`Utilities`StyleTables`"
, "Utilities`FilterOptions`"] ;

NumberLine::usage=
"NumberLine[start_,stop_,opts___Rule] creates a number line drawn between 
start and stop with properties determined by its options."

NumberLineIntervalColor::usage =
"NumberLineIntervalColor is an option of NumberLine that specifies the colour
to be applied to the interval or band.  The value is a list of styles that
can be applied to graphics objects.\n
e.g. NumberLineIntervalColor -> RGBColor[0,1,0]    specifies that the interval
is to be made in green."

NumberLineIntervalLength::usage =
"NumberLineIntervalLength is an option of NumberLine which specifies the length
of the interval or banding on the NumberLine.\n
e.g. NumberLineIntervalLength->5 specifies that the band is 5 units long.\n
NumberLineIntervalLength->None specifies that there is no band"

NumberLineIntervalStart::usage =
"NumberLineIntervalStart is an option of NumberLine that specifies where the 
first interval or band starts.\n
e.g. NumberLineIntervalStart->0 specifies that the first band starts at 0.\n
NumberLineIntervalStart->Automatic specifies that the first band starts at the
beginning of the number line."

NumberLineNumbers::usage =
"NumberLineNumbers is an option of NumberLine that specifies the numbers to
be displayed on the number line.\n
e.g. NumberLineNumbers->Range[1,5] displays the integers between 1 and 5.\n
NumberLineNumbers->Automatic puts the integers on the number line.\n
NumberLineNumbers->None puts no numbers on the number line."

Options[NumberLine] = 
  { NumberLineIntervalColor->RGBColor[0,1,0]
  , NumberLineIntervalLength->None
  , NumberLineIntervalStart->Automatic
  , NumberLineNumbers->Automatic
  } ;

Begin["`Private`"] ;

NumberLine[start_,stop_,opts___Rule] :=
  Module[{ i 
    , intervalColor, intervalLength, intervalStart, lineNumbers 
    , lineNumbersStyle, numOption 
    }
  ,
    { intervalColor , intervalLength, intervalStart , lineNumbers 
    } = 
      { NumberLineIntervalColor, NumberLineIntervalLength 
      , NumberLineIntervalStart, NumberLineNumbers
      } /. {opts} /. Options[NumberLine] ;
    lineNumbers = 
      lineNumbers //. { Automatic -> Range[Ceiling[start],Floor[stop]] } ;
    If[ intervalStart === Automatic
    , intervalStart = start
    ] ;
    If[ lineNumbers === None
    , textList = {} 
    , Do[
        If[ (Head[lineNumbers[[i]] ] === List) && NumberQ[Last[lineNumbers[[i]]]]
        , lineNumbers[[i]] = { lineNumbers[[i]] , {} } 
        ]
      , { i , Length[lineNumbers] }
      ] ;
      { lineNumbers , lineNumbersStyle } =
        StyleTableExpand[lineNumbers, StyleTableOrdering -> Append ] ;
      Do[ 
        numOption[i] =
          {FilterOptions[Text,lineNumbersStyle[[i,2]]]} 
      , { i , Length[lineNumbers] }
      ] ;
      textList = 
        Table[
          Apply[ Text
          , Join[
              { lineNumbers[[i]],{lineNumbers[[i]],0} }
            , numOption[i]
            ]
          ] 
        , { i , Length[lineNumbers] }
        ] 
    ] ; 
    intervals = 
      Which[ 
        intervalLength === None
        , {}
      , (Head[intervalColor] === List) && (Length[intervalColor] > 0)
        , Table[
            Table[ 
              { intervalColor[[j]] 
              , Rectangle[{i+intervalLength*(j-1),-1},{i+intervalLength*j,1}]
              }
            , { j , Length[intervalColor] }
            ] 
          , { i , intervalStart,stop,intervalLength*Length[intervalColor] }
          ]  
      , True
        , Join[ {intervalColor }
          , Table[
              Rectangle[{i,-1},{i+intervalLength,1}]
            , {i,intervalStart,stop,intervalLength*2}
            ]
          ]
      ] ;
    Show[
      Graphics[
        { intervals, textList }
      ]
    , FilterOptions[Graphics,opts]
    , PlotRange->{{start,stop},{-1.1,1.1}} 
    ]
  ];

End[] ;

EndPackage[] ;