(*  This file contains functions for creating and manipulating number squares. 
    version 1  06-07-99
*)

BeginPackage["PrimaryMath`PursuitPlots`PursuitPlots`","Utilities`FilterOptions`"] ;

PursuitCurve::usage=
"PursuitCurve[{{startPoint_,targetLocation_}..},{t_,start_,stop_,dt_},opts___Rule] 
computes the paths of each of 
the pursuers.  startPoint represents the starting point of each pursuer, and targetLocation specifies 
the location of the target that that pursuer is pursuing.  targetLocation can be a function which 
returns the location at the time t_, or it can be Pursuer[i], where i is an integer denoting which of the 
other pursuers is being pursued.  t_ is the time variable, and the curves are computed between
t=start and t=stop, with the path directions being recalculated after each time interval dt.\n
The result returned is of the form { pursuitPaths , pursuitVectors }, where pursuitCurves is a list 
of the paths that the pursuers follow, and pursuitVectors is a list of the vectors 
the pursuers have set their course by."

Options[PursuitCurve] = 
  { PursuerSpeed -> 1 
  } ;

Pursuer::usage=
"Pursuer[j] (where j is an integer) is a term used as the targetLocation in the ith part 
of the PursuerParameters of 
PursuitCurve to denote that the ith pursuer is chasing the jth Pursuer."

PursuerSpeed::usage=
"PursuerSpeed is an option of PursuitCurve which specifies the speed of the pursuer(s).  
It can be a number, 
in which case it is applied to all the pursuers.  
Alternatively it can be a list of numbers which represent
speeds of the individual pursuers.  
If the length of the list of speeds is less than number of pursuers, then
the list is cycled to give the speeds of all the pursuers.  
For example, if the list has four numbers, and there
are six pursuers, then the fifth pursuer's speed is set to the first element of the list, 
and the sixth pursuer's speed is set to the second element of the list." 

Begin["`Private`"] ;

PursuitCurve[pursuerParameters:{{_?NumberQ,_?NumberQ},_},{t_,start_,stop_,dt_},opts___Rule]:=
  PursuitCurve[{pursuerParameters},{t,start,stop,dt},opts] ;

PursuitCurve[pursuerParameters:{{{_?NumberQ,_?NumberQ},_}..},{t_,start_,stop_,dt_},opts___Rule] :=
  Module[{ directionVector , i , j , newPoints 
    , numberOfPursuers , points , pursuerSpeed , pursuitCurves
    , pursuitVectors, relativeLocationVector , target 
    }
  ,
    { pursuerSpeed } =
      { PursuerSpeed } /. {opts} /. Options[PursuitCurve] ;
    numberOfPursuers = Length[pursuerParameters] ;
    (* Set up the speed vector with the speeds of the pursuers *)
    speed = 
      Which[ 
        NumberQ[pursuerSpeed]
        , Table[pursuerSpeed,{numberOfPursuers}]
      , numberOfPursuers > Length[pursuerSpeed]
        , Flatten[Table[pursuerSpeed,{i,1,numberOfPursuers,Length[pursuerSpeed]}]];
      , numberOfPursuers < Length[pursuerSpeed]
        , pursuerSpeed
      ] ;
    points = Table[pursuerParameters[[i,1]],{i,numberOfPursuers}] ;
    newPoints = points ;  (* Initialise newPoints as an array of the same size as points *)

    (* Initialise pursuitVectors matrix *)
    pursuitVectors =
      Table[Null,{i,numberOfPursuers},{t,start,stop,dt}] ;

    j = 0 ;    
    pursuitCurves =
      Map[ Line
      , Transpose[ 
          Join[ {points}
          , Table[
              j++ ;
              Table[
                target =
                  If[ MatchQ[pursuerParameters[[i,2]] , Pursuer[_] ]
                  , points[[ pursuerParameters[[i,2,1]] ]]
                  , pursuerParameters[[i,2]]  
                  ] ;
                relativeLocationVector = target - points[[i]] ;
                directionVector = relativeLocationVector / Sqrt[Apply[Plus,Map[(#^2)&,relativeLocationVector]]] ;
                newPoints[[i]] = points[[i]] + speed[[i]] * dt * directionVector ;
                pursuitVectors[[i,j]] = Line[ {points[[i]] , target} ] ;
              , { i , numberOfPursuers }
              ] ;
              points = newPoints
            , { t , start , stop , dt }
            ]
          ] 
        ]
      ] ;
    { pursuitCurves , pursuitVectors }
  ] ; 

End[] ;

EndPackage[] ;