(* This package contains routines for rotating and translating 2D and 3D shapes
     version 1  15-05-99
*)

BeginPackage["PrimaryMath`Shapes`ShapeTranslations`","Geometry`Rotations`"] ;

MirrorShape::usage="MirrorShape[expression_,normalVector_,opts___Rule] mirrors the 
objects in expression by
reflecting them back to the other side of the origin through the direction 
defined by normalVector.\n
If normalVector is 3-dimensional, then a 3 dimensional reflection is done.  If
normalVector is 2-dimensional, then a 2 dimensional reflection is done."   

Options[MirrorShape] = {Shape2DOrigin -> {0,0} , Shape3DOrigin -> {0,0,0} } ;

MoveShape::usage="MoveShape[expression_,{x_,y_,z_}] moves the 3D objects in expression 
by the offset {x,y,z}.\n
MoveShape[expression_,{x_,y_}] moves the 2D objects in expression by the offset {x,y}."

ScaleShape::usage="ScaleShape[expression_,{xScale_,yScale_,zScale_}] scales the 3D objects 
in expression by the scale factors {xScale,yScale,zScale}, with respect to the 
origin specified by Shape3DOrigin.\n
ScaleShape[expression_,{xScale_,yScale_}] scales the 2D objects in expression 
by the scale factors {xScale,yScale} with the origin specified by Shape2DOrigin." 

Options[ScaleShape] = {Shape2DOrigin -> {0,0} , Shape3DOrigin -> {0,0,0} } ;

TurnShape::usage="TurnShape[expression_,angle_,axis_,opts___Rule] rotates the 3D objects in expression by
angle around axis].\n
TurnShape[expression_,theta_,opts___Rule] rotates the 2D objects in expression by the angle theta."

Options[TurnShape] = {Shape2DOrigin -> {0,0} , Shape3DOrigin -> {0,0,0} } ;

Shape2DOrigin::usage="Shape2DOrigin is an option of MirrorShape, ScaleShape and TurnShape
which specifies the local origin (in 2 dimensions) with respect to which 
the shape is adjusted."

Shape3DOrigin::usage="Shape3DOrigin is an option of MirrorShape, ScaleShape and TurnShape
which specifies the local origin (in 3 dimensions) with respect to which 
the shape is adjusted." 

Begin["`Private`"] ;



MirrorShape[expression_ , normalVector_List , opts___Rule ] :=
  Module[ { line , mirrorMapping , normalisedVector, origin , p , point , poly }
  , { origin } = { Shape2DOrigin } /. {opts} /. Options[MirrorShape] ;
    normalisedVector = 
      normalVector / 
        Sqrt[ normalVector[[1]]^2 + normalVector[[2]]^2 ] ;

    mirrorMapping[p_] :=  p-(2*normalisedVector*(normalisedVector . ( p - origin ))) ;

    expression /.
      { poly:Polygon[_] :> Map[ mirrorMapping , poly , {2} ]
      , line:Line[_] :> Map[ mirrorMapping , line , {2} ]
      , point:Point[_] :> Map[ mirrorMapping , point , {1} ]
      }
  ] /; ( Length[normalVector] === 2 )

MirrorShape[expression_ , normalVector_List , opts___Rule ] :=
  Module[ { line , mirrorMapping , normalisedVector , origin , p , point , poly }
  , { origin } = { Shape3DOrigin } /. {opts} /. Options[MirrorShape] ;
    normalisedVector = 
      normalVector /
        Sqrt[ normalVector[[1]]^2 + normalVector[[2]]^2 + normalVector[[3]]^2 ] ;

    mirrorMapping[p_] := p-(2*normalisedVector*(normalisedVector . ( p - origin ))) ;

    expression /.
      { poly:Polygon[_] :> Map[ mirrorMapping , poly , {2} ]
      , line:Line[_] :> Map[ mirrorMapping , line , {2} ]
      , point:Point[_] :> Map[ mirrorMapping , point , {1} ]
      }
  ] /; ( Length[normalVector] === 3 )

MoveShape[ expression_ , vector_ ] :=
  Module[ { line , moveMapping , p , point , poly } 
  ,  moveMapping[p_] := p+vector ;
     expression /. 
      { poly:Polygon[_] :> Map[ moveMapping , poly , {2} ]
      , line:Line[_] :> Map[ moveMapping , line , {2} ]
      , point:Point[_] :> Map[ moveMapping , point , {1} ]
      }
  ] /; ( Length[vector] === 2 )

MoveShape[ expression_ , vector_ ] :=
  Module[ { line , moveMapping , p , point , poly } 
  ,  moveMapping[p_] := p+vector ;
     expression /. 
      { poly:Polygon[_] :> Map[ moveMapping , poly , {2} ]
      , line:Line[_] :> Map[ moveMapping , line , {2} ]
      , point:Point[_] :> Map[ moveMapping , point , {1} ]
      }
  ] /; ( Length[vector] === 3 )

ScaleShape[ expression_ , vector_ , opts___Rule ] :=
  Module[ { line , scaleMapping , origin , p , point , poly } 
  ,  { origin } = { Shape2DOrigin } /. {opts} /. Options[ScaleShape] ;
     scaleMapping[p_] := origin + (vector*(p-origin)) ;
     expression /. 
      { poly:Polygon[_] :> Map[ scaleMapping , poly , {2} ]
      , line:Line[_] :> Map[ scaleMapping , line , {2} ]
      , point:Point[_] :> Map[ scaleMapping , point , {1} ]
      }
  ] /; ( Length[vector] === 2 )

ScaleShape[ expression_ , vector_ , opts___Rule ] :=
  Module[ { line , origin , p , point , poly , scaleMapping } 
  ,  { origin } = { Shape3DOrigin } /. {opts} /. Options[ScaleShape] ;
     scaleMapping[p_] := (origin + (vector*(p-origin))) ;
     expression /. 
      { poly:Polygon[_] :> Map[ scaleMapping , poly , {2} ]
      , line:Line[_] :> Map[ scaleMapping , line , {2} ]
      , point:Point[_] :> Map[ scaleMapping , point , {1} ]
      }
  ] /; ( Length[vector] === 3 )

TurnShape[expression_ , angle_ , opts___Rule ] :=
  Module[ { line , localPoint , normalVector 
    , origin , p , point , poly , turnMapping } 
  , { origin } = { Shape2DOrigin } /. {opts} /. Options[TurnShape] ;
 
    (* Define a function for calculating the turn mapping for an individual point *)
    turnMapping[p_] :=
      (
        localPoint = p - origin ;
        normalVector = { -localPoint[[2]] , localPoint[[1]] } ;
        origin + Cos[angle] localPoint + Sin[angle] normalVector 
      ) ;
    expression /. 
      { poly:Polygon[_] :> Map[ turnMapping , poly , {2} ]
      , line:Line[_] :> Map[ turnMapping , line , {2} ]
      , point:Point[_] :> Map[ turnMapping , point , {1} ]
      }
  ] ;



TurnShape[ expression_ , angle_,axis_,opts___Rule ] :=
  Module[ { axisOffset , axisToPoint, axisToNormal , line , localPoint 
    , normalisedAxis , origin , p , point , poly , turnMapping }
  , { origin } = { Shape3DOrigin } /. {opts} /. Options[TurnShape] ; 
    normalisedAxis = axis / Sqrt[axis[[1]]^2 + axis[[2]]^2 + axis[[3]]^2] ;

    (* Define a function for calculating the turn mapping for an individual point *)
    turnMapping[p_] :=
      (  
        (* subtract the local origin *)
        localPoint = p-origin ;
        (* Calculate the distance along the axis corresponding to the
           normal plane holding localPosition *)
        axisOffset = normalisedAxis ( localPoint . normalisedAxis ) ;
        (* Compute the vector in the plane to localPoint *)
        axisToPoint = localPoint - axisOffset ;
        (* Compute the normal vector in the plane *)
        axisToNormal = Cross[normalisedAxis,axisToPoint] ;
        (* Return the answer *)
        axisOffset + Cos[angle] axisToPoint + Sin[angle] axisToNormal
      ) ;

    expression /. 
      { poly:Polygon[_] :> Map[ turnMapping , poly,{2}]
      ,  line:Line[_] :> Map[ turnMapping , line,{2}]
      ,  point:Point[_] :> Map[ turnMapping ,point,{1}]
      }
  ] ;



End[] ;

EndPackage[] ;
