(*  This file contains functions for expanding options.  It is used by other
    packages.
    version 1  29-02-00
*)

BeginPackage["PrimaryMath`Utilities`StyleTables`"] ;


DefaultStyles::usage =
"DefaultStyles is an option of StyleTableExpand which specifies what the default styles are that 
should be applied to the objects."

StyleTableExpand::usage =
"StyleTableExpand[ {({object_ || objectList_List , style_ || styleList_List }||object_)..} , 
opts___Rule ]
is a function which rearranges lists of objects and styles associated
with those objects into a list of the form {{object[i],assocatedStyles_List}..}, where i
indexes all the objects.  The answer returned is in the form { objectList, objectAndStyleList }
where objectList is a list of objects passed to StyleTableExpand, 
and objectAndStyleList = {{object[i],associatedStyles_List}..}.\n
If no styles are assigned to an object, it is passed back with an empty list of associated
styles.  However default styles can be assigned with the DefaultStyles option."

Options[StyleTableExpand] =
  { DefaultStyles -> {}
  , StyleTableOrdering -> Append } ;

StyleTableOrdering::usage =
"StyleTableOrdering is an option of StyleTableExpand that indicates how
the styles are to be ordered for the individual entries.\n
StyleTableOrdering->Append specifies that any later style definitions are to be added to the
end of the list of styles for each value.\n
StyleTableOrdering->Prepend specifies that any later style definitions are to be added to
the front of the list of styles for each value."


Begin["`Private`"] ;

StyleTableExpand[ inputStyleTable_List ,  opts___Rule ] :=
  Module[ { defaultStyleList , expandedStyleTable
       , i , j , objectList = {} , pendTo , styleList , styleTable = inputStyleTable
       , styleTableOrdering
     }
  , 
    (* Read in and set up the options *)
    { defaultStyleList , styleTableOrdering } =
      { DefaultStyles , StyleTableOrdering } /. { opts } /. Options[StyleTableExpand] ;
    If[ styleTableOrdering === Prepend
    , pendTo[ objectL_ , styles_ ] :=
        Set[objectL,Join[styles,objectL]]  
    , pendTo[ objectL_ , styles_ ] :=
        Set[objectL,Join[objectL,styles]]
    ] ;
    SetAttributes[pendTo,HoldFirst] ;
    (* If elements of the list are not lists, assume that they are values to
       be assigned the default styles - enclose them in lists *)
    Do[
      If[ Head[styleTable[[i]]] =!= List
      , styleTable[[i]] = { styleTable[[i]] , {} } 
      ]
    , { i , Length[styleTable] }
    ] ;
    (* Make sure that every value and style is enclosed in a list *)
    Do[
      If[Head[styleTable[[i,j]]] =!= List
      , styleTable[[i,j]] = {styleTable[[i,j]]}
      ]
    , {i,Length[styleTable]} 
    , {j , 2 }
    ] ;
    (* Now extract a list of objects, and assign a default list to each object, which
       later will hold the styles. *)
    Do[
      If[ !MemberQ[objectList,styleTable[[i,1,j]] ]
      , AppendTo[objectList,styleTable[[i,1,j]] ] ;
        styleList[ styleTable[[i,1,j]] ] = defaultStyleList
      ]  
    , { i , Length[styleTable] }
    , { j , Length[ styleTable[[i,1]] ] }
    ] ;
    (* Next fill the lists with the styles associated with the objects *)
    Do[
      pendTo[ styleList[Evaluate[styleTable[[i,1,j]]] ] , styleTable[[i,2]] ]
    , { i , Length[styleTable] }
    , { j , Length[styleTable[[i,1]] ] }
    ] ;
    (* Combine the extracted styles into one long list, together with their objects *)
    expandedStyleTable =
      Table[ { objectList[[i]] , styleList[objectList[[i]] ] }
      , { i , Length[objectList] }
      ] ;
    (* Return result *)
    { objectList , expandedStyleTable }
  ] ;



End[] ;

EndPackage[] ;